@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# This is a kernel build script for Fedora 14's 2.6.35 kernel.
+# This is a kernel build script for Fedora 12's 2.6.32 kernel.
 #
 
 die () {
@@ -10,17 +10,17 @@ die () {
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 
-if [ ! -r kernel-2.6.35.10-74.fc14.src.rpm ]
+if [ ! -r kernel-2.6.32.26-175.fc12.src.rpm ]
 then
-    wget http://ftp.riken.jp/Linux/fedora/updates/14/SRPMS/kernel-2.6.35.10-74.fc14.src.rpm || die "Can't download source package."
+    wget http://ftp.riken.jp/Linux/fedora/updates/12/SRPMS/kernel-2.6.32.26-175.fc12.src.rpm || die "Can't download source package."
 fi
-rpm --checksig kernel-2.6.35.10-74.fc14.src.rpm || die "Can't verify signature."
-rpm -ivh kernel-2.6.35.10-74.fc14.src.rpm || die "Can't install source package."
+rpm --checksig kernel-2.6.32.26-175.fc12.src.rpm || die "Can't verify signature."
+rpm -ivh kernel-2.6.32.26-175.fc12.src.rpm || die "Can't install source package."
 
 cd /root/rpmbuild/SOURCES/ || die "Can't chdir to /root/rpmbuild/SOURCES/ ."
-if [ ! -r ccs-patch-1.8.0-20110207.tar.gz ]
+if [ ! -r ccs-patch-1.8.1-20110411.tar.gz ]
 then
-    wget -O ccs-patch-1.8.0-20110207.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.0-20110207.tar.gz' || die "Can't download patch."
+    wget -O ccs-patch-1.8.1-20110411.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.1-20110411.tar.gz' || die "Can't download patch."
 fi
 
 cd /root/rpmbuild/SPECS/ || die "Can't chdir to /root/rpmbuild/SPECS/ ."
@@ -28,16 +28,16 @@ cp -p kernel.spec ccs-kernel.spec || die "Can't copy spec file."
 patch << "EOF" || die "Can't patch spec file."
 --- ccs-kernel.spec
 +++ ccs-kernel.spec
-@@ -23,7 +23,7 @@
- #
- # (Uncomment the '#' and both spaces below to set the buildid.)
+@@ -22,7 +22,7 @@
+ # appended to the full kernel version.
  #
+ # (Uncomment the '#' and the first two spaces below to set buildid.)
 -# % define buildid .local
-+%define buildid _tomoyo_1.8.0p3
++%define buildid _tomoyo_1.8.1p1
  ###################################################################
  
  # The buildid can also be specified on the rpmbuild command line
-@@ -412,6 +412,11 @@
+@@ -429,6 +429,11 @@
  # to versions below the minimum
  #
  
@@ -49,7 +49,16 @@ patch << "EOF" || die "Can't patch spec file."
  #
  # First the general kernel 2.6 required versions as per
  # Documentation/Changes
-@@ -471,7 +476,7 @@
+@@ -464,7 +469,7 @@
+ # Packages that need to be installed before the kernel is, because the %post
+ # scripts use them.
+ #
+-%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, kernel-firmware >= %{rpmversion}-%{pkg_release}, grubby >= 7.0.4-1
++%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, grubby >= 7.0.4-1
+ %if %{with_dracut}
+ %define initrd_prereq  dracut >= 002 xorg-x11-drv-ati-firmware
+ %else
+@@ -500,7 +505,7 @@
  AutoProv: yes\
  %{nil}
  
@@ -58,27 +67,27 @@ patch << "EOF" || die "Can't patch spec file."
  Group: System Environment/Kernel
  License: GPLv2
  URL: http://www.kernel.org/
-@@ -900,7 +905,7 @@
+@@ -950,7 +955,7 @@
+ Provides: kernel-devel-uname-r = %{KVERREL}%{?1:.%{1}}\
  AutoReqProv: no\
  Requires(pre): /usr/bin/find\
- Requires: perl\
 -%description -n kernel%{?variant}%{?1:-%{1}}-devel\
 +%description -n ccs-kernel%{?variant}%{?1:-%{1}}-devel\
  This package provides kernel headers and makefiles sufficient to build modules\
  against the %{?2:%{2} }kernel package.\
  %{nil}
-@@ -1498,6 +1503,10 @@
- 
- # END OF PATCH APPLICATIONS
- 
-+# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.8.0-20110207.tar.gz
-+patch -sp1 < patches/ccs-patch-2.6.35-fedora-14.diff
-+
+@@ -1568,6 +1573,10 @@
+ # END OF PATCH APPLICATIONS ====================================================
  %endif
  
++# TOMOYO Linux
++tar -zxf %_sourcedir/ccs-patch-1.8.1-20110411.tar.gz
++patch -sp1 < patches/ccs-patch-2.6.32-fedora-12.diff
++
  # Any further pre-build tree manipulations happen here.
-@@ -1526,6 +1535,9 @@
+ 
+ chmod +x scripts/checkpatch.pl
+@@ -1594,6 +1603,9 @@
  for i in *.config
  do
    mv $i .config
@@ -102,7 +111,7 @@ sleep 30
 patch << "EOF" || die "Can't patch spec file."
 --- /root/rpmbuild/SPECS/ccs-kernel.spec
 +++ /root/rpmbuild/SPECS/ccs-kernel.spec
-@@ -220,7 +220,7 @@
+@@ -224,7 +224,7 @@
  
  # kernel-PAE is only built on i686.
  %ifarch i686
