@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/file.c
  *
- * Copyright (C) 2005-2010  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.0   2010/11/11
+ * Version: 1.8.1+   2011/04/11
  */
 
 #include "internal.h"
@@ -20,6 +20,12 @@
 #endif
 #define ACC_MODE(x) ("\000\004\002\006"[(x)&O_ACCMODE])
 
+#if defined(RHEL_MAJOR) && RHEL_MAJOR == 6
+/* RHEL6 passes unmodified flags since 2.6.32-71.14.1.el6 . */
+#undef ACC_MODE
+#define ACC_MODE(x) ("\004\002\006"[(x)&O_ACCMODE])
+#endif
+
 #endif
 
 /* Mapping table from "enum ccs_path_acl_index" to "enum ccs_mac_index". */
@@ -352,7 +358,7 @@ static bool ccs_same_path_acl(const struct ccs_acl_info *a,
  *
  * @a:         Pointer to "struct ccs_acl_info".
  * @b:         Pointer to "struct ccs_acl_info".
- * @is_delete: True for @a &= ~@b, false for @a |= @b. 
+ * @is_delete: True for @a &= ~@b, false for @a |= @b.
  *
  * Returns true if @a is empty, false otherwise.
  */
@@ -421,7 +427,7 @@ static bool ccs_same_mkdev_acl(const struct ccs_acl_info *a,
  *
  * @a:         Pointer to "struct ccs_acl_info".
  * @b:         Pointer to "struct ccs_acl_info".
- * @is_delete: True for @a &= ~@b, false for @a |= @b. 
+ * @is_delete: True for @a &= ~@b, false for @a |= @b.
  *
  * Returns true if @a is empty, false otherwise.
  */
@@ -494,7 +500,7 @@ static bool ccs_same_path2_acl(const struct ccs_acl_info *a,
  *
  * @a:         Pointer to "struct ccs_acl_info".
  * @b:         Pointer to "struct ccs_acl_info".
- * @is_delete: True for @a &= ~@b, false for @a |= @b. 
+ * @is_delete: True for @a &= ~@b, false for @a |= @b.
  *
  * Returns true if @a is empty, false otherwise.
  */
@@ -702,7 +708,7 @@ static int __ccs_open_permission(struct dentry *dentry, struct vfsmount *mnt,
 	idx = ccs_read_lock();
 	if (acc_mode && ccs_init_request_info(&r, CCS_MAC_FILE_OPEN)
 	    != CCS_CONFIG_DISABLED) {
-		if (!buf.name && !ccs_get_realpath(&buf, dentry, mnt)) {
+		if (!ccs_get_realpath(&buf, dentry, mnt)) {
 			error = -ENOMEM;
 			goto out;
 		}
@@ -964,7 +970,7 @@ static bool ccs_same_path_number_acl(const struct ccs_acl_info *a,
  *
  * @a:         Pointer to "struct ccs_acl_info".
  * @b:         Pointer to "struct ccs_acl_info".
- * @is_delete: True for @a &= ~@b, false for @a |= @b. 
+ * @is_delete: True for @a &= ~@b, false for @a |= @b.
  *
  * Returns true if @a is empty, false otherwise.
  */
@@ -1129,7 +1135,7 @@ static int __ccs_chown_permission(struct dentry *dentry,
  * @file: Pointer to "struct file".
  * @cmd:  Command number.
  * @arg:  Value for @cmd.
- * 
+ *
  * Returns 0 on success, negative value otherwise.
  */
 static int __ccs_fcntl_permission(struct file *file, unsigned int cmd,
@@ -1138,11 +1144,14 @@ static int __ccs_fcntl_permission(struct file *file, unsigned int cmd,
 	if (!(cmd == F_SETFL && ((arg ^ file->f_flags) & O_APPEND)))
 		return 0;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
-	/* 01 means "write". */
-	return __ccs_open_permission(file->f_dentry, file->f_vfsmnt, 01);
+	return __ccs_open_permission(file->f_dentry, file->f_vfsmnt,
+				     O_WRONLY | (arg & O_APPEND));
+#elif defined(RHEL_MAJOR) && RHEL_MAJOR == 6
+	return __ccs_open_permission(file->f_dentry, file->f_vfsmnt,
+				     O_WRONLY | (arg & O_APPEND));
 #else
-	/* 02 means "write". */
-	return __ccs_open_permission(file->f_dentry, file->f_vfsmnt, 02);
+	return __ccs_open_permission(file->f_dentry, file->f_vfsmnt,
+				     (O_WRONLY + 1) | (arg & O_APPEND));
 #endif
 }
 
@@ -1413,8 +1422,7 @@ static int __ccs_open_exec_permission(struct dentry *dentry,
 				      struct vfsmount *mnt)
 {
 	return (ccs_current_flags() & CCS_TASK_IS_IN_EXECVE) ?
-		/* 01 means "read". */
-		__ccs_open_permission(dentry, mnt, 01) : 0;
+		__ccs_open_permission(dentry, mnt, O_RDONLY + 1) : 0;
 }
 
 /**
@@ -1427,8 +1435,7 @@ static int __ccs_open_exec_permission(struct dentry *dentry,
  */
 static int __ccs_uselib_permission(struct dentry *dentry, struct vfsmount *mnt)
 {
-	/* 01 means "read". */
-	return __ccs_open_permission(dentry, mnt, 01);
+	return __ccs_open_permission(dentry, mnt, O_RDONLY + 1);
 }
 
 #endif
