@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.0+   2011/02/07
+ * Version: 1.8.1   2011/04/01
  */
 
 #ifndef _SECURITY_CCSECURITY_INTERNAL_H
@@ -16,7 +16,9 @@
 #include <linux/mm.h>
 #include <linux/utime.h>
 #include <linux/file.h>
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 38)
 #include <linux/smp_lock.h>
+#endif
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/slab.h>
@@ -292,7 +294,7 @@ static inline void list_add_rcu(struct list_head *new, struct list_head *head)
  * @pos:    The type * to use as a loop cursor.
  * @head:   The head for your list.
  * @member: The name of the list_struct within the struct.
- * @ss:     Pointer to "struct srcu_struct".  
+ * @ss:     Pointer to "struct srcu_struct".
  *
  * As of 2.6.36, this macro is not provided because only TOMOYO wants it.
  */
@@ -537,7 +539,7 @@ enum ccs_mac_index {
 	CCS_MAX_MAC_INDEX
 };
 
-/* Index numbers for /proc/ccs/meminfo interface. */
+/* Index numbers for /proc/ccs/stat interface. */
 enum ccs_memory_stat_type {
 	CCS_MEMORY_POLICY,
 	CCS_MEMORY_AUDIT,
@@ -664,7 +666,6 @@ enum ccs_proc_interface_index {
 	CCS_EXCEPTIONPOLICY,
 	CCS_DOMAIN_STATUS,
 	CCS_PROCESS_STATUS,
-	CCS_MEMINFO,
 	CCS_STAT,
 	CCS_AUDIT,
 	CCS_VERSION,
@@ -779,7 +780,7 @@ enum ccs_value_type {
 		       __GFP_NORETRY | __GFP_NOMEMALLOC)
 
 /* Size of read buffer for /proc/ccs/ interface. */
-#define CCS_MAX_IO_READ_QUEUE 32
+#define CCS_MAX_IO_READ_QUEUE 64
 
 /* Structure definition for internal use. */
 
@@ -1314,50 +1315,52 @@ struct ccs_acl_param {
 	bool is_delete;
 };
 
-/* Structure for reading/writing policy via /proc interfaces. */
+/* Structure for reading/writing policy via /proc/ccs/ interfaces. */
 struct ccs_io_buffer {
 	void (*read) (struct ccs_io_buffer *);
 	int (*write) (struct ccs_io_buffer *);
 	int (*poll) (struct file *file, poll_table *wait);
 	/* Exclusive lock for this structure.   */
 	struct mutex io_sem;
-	/* Index returned by ccs_lock().        */
-	int reader_idx;
 	char __user *read_user_buf;
-	int read_user_buf_avail;
+	size_t read_user_buf_avail;
 	struct {
 		struct list_head *domain;
 		struct list_head *group;
 		struct list_head *acl;
-		int avail;
-		int step;
-		int query_index;
+		size_t avail;
+		unsigned int step;
+		unsigned int query_index;
 		u16 index;
 		u16 cond_index;
-		u8 group_index;
+		u8 acl_group_index;
 		u8 cond_step;
 		u8 bit;
 		u8 w_pos;
 		bool eof;
 		bool print_this_domain_only;
-		bool print_execute_only;
+		bool print_transition_related_only;
 		bool print_cond_part;
 		const char *w[CCS_MAX_IO_READ_QUEUE];
 	} r;
 	struct {
 		struct ccs_domain_info *domain;
-		int avail;
+		size_t avail;
 	} w;
 	/* Buffer for reading.                  */
 	char *read_buf;
 	/* Size of read buffer.                 */
-	int readbuf_size;
+	size_t readbuf_size;
 	/* Buffer for writing.                  */
 	char *write_buf;
 	/* Size of write buffer.                */
-	int writebuf_size;
-	/* One of values in "enum ccs_proc_interface_index". */
-	u8 type;
+	size_t writebuf_size;
+	/* Type of interface. */
+	enum ccs_proc_interface_index type;
+	/* Users counter protected by ccs_io_buffer_list_lock. */
+	u8 users;
+	/* List for telling GC not to kfree() elements. */
+	struct list_head list;
 };
 
 /* Structure for /proc/ccs/profile interface. */
@@ -1434,15 +1437,12 @@ int ccs_delete_domain(char *data);
 int ccs_env_perm(struct ccs_request_info *r, const char *env);
 int ccs_get_path(const char *pathname, struct path *path);
 int ccs_init_request_info(struct ccs_request_info *r, const u8 index);
-int ccs_lock(void);
 int ccs_open_control(const u8 type, struct file *file);
 int ccs_parse_ip_address(char *address, u16 *min, u16 *max);
 int ccs_path_permission(struct ccs_request_info *r, u8 operation,
 			const struct ccs_path_info *filename);
 int ccs_poll_control(struct file *file, poll_table *wait);
 int ccs_poll_log(struct file *file, poll_table *wait);
-int ccs_read_control(struct file *file, char __user *buffer,
-		     const int buffer_len);
 int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
 	__attribute__ ((format(printf, 2, 3)));
 int ccs_symlink_path(const char *pathname, struct ccs_path_info *name);
@@ -1459,8 +1459,6 @@ int ccs_update_policy(struct ccs_acl_head *new_entry, const int size,
 					       const struct ccs_acl_head *));
 int ccs_write_aggregator(char *data, const bool is_delete);
 int ccs_write_capability(struct ccs_acl_param *param);
-int ccs_write_control(struct file *file, const char __user *buffer,
-		      const int buffer_len);
 int ccs_write_file(struct ccs_acl_param *param);
 int ccs_write_group(char *data, const bool is_delete, const u8 type);
 int ccs_write_inet_network(struct ccs_acl_param *param);
@@ -1470,7 +1468,10 @@ int ccs_write_reserved_port(char *data, const bool is_delete);
 int ccs_write_transition_control(char *data, const bool is_delete,
 				 const u8 type);
 int ccs_write_unix_network(struct ccs_acl_param *param);
-size_t ccs_del_condition(struct list_head *element);
+ssize_t ccs_read_control(struct file *file, char __user *buffer,
+			 const size_t buffer_len);
+ssize_t ccs_write_control(struct file *file, const char __user *buffer,
+			  const size_t buffer_len);
 struct ccs_condition *ccs_get_condition(char *condition);
 struct ccs_domain_info *ccs_assign_domain(const char *domainname,
 					  const u8 profile, const u8 group,
@@ -1485,10 +1486,12 @@ void ccs_check_acl(struct ccs_request_info *r,
 		   bool (*check_entry) (struct ccs_request_info *,
 					const struct ccs_acl_info *));
 void ccs_convert_time(time_t time, struct ccs_time *p);
+void ccs_del_condition(struct list_head *element);
 void ccs_fill_path_info(struct ccs_path_info *ptr);
 void ccs_get_attributes(struct ccs_obj_info *obj);
 void ccs_memory_free(const void *ptr, size_t size);
 void ccs_normalize_line(unsigned char *buffer);
+void ccs_notify_gc(struct ccs_io_buffer *head, const bool is_register);
 void ccs_print_ipv4(char *buffer, const int buffer_len, const u32 min_ip,
 		    const u32 max_ip);
 void ccs_print_ipv6(char *buffer, const int buffer_len,
@@ -1499,22 +1502,24 @@ void ccs_print_ulong(char *buffer, const int buffer_len,
 void ccs_put_name_union(struct ccs_name_union *ptr);
 void ccs_put_number_union(struct ccs_number_union *ptr);
 void ccs_read_log(struct ccs_io_buffer *head);
-void ccs_run_gc(void);
 void ccs_transition_failed(const char *domainname);
-void ccs_unlock(const int idx);
 void ccs_update_stat(const u8 index);
 void ccs_warn_oom(const char *function);
 void ccs_write_log(struct ccs_request_info *r, const char *fmt, ...)
 	__attribute__ ((format(printf, 2, 3)));
 void ccs_write_log2(struct ccs_request_info *r, int len, const char *fmt,
 		    va_list args);
+#ifdef CONFIG_CCSECURITY_USE_BUILTIN_POLICY
+void __init ccs_load_builtin_policy(void);
+#endif
 
 /* Variable definition for internal use. */
 
 extern bool ccs_policy_loaded;
 extern const char * const ccs_condition_keyword[CCS_MAX_CONDITION_KEYWORD];
 extern const char * const ccs_dif[CCS_MAX_DOMAIN_INFO_FLAGS];
-extern const char * const ccs_mac_keywords[CCS_MAX_MAC_INDEX + CCS_MAX_MAC_CATEGORY_INDEX];
+extern const char * const ccs_mac_keywords[CCS_MAX_MAC_INDEX
+					   + CCS_MAX_MAC_CATEGORY_INDEX];
 extern const char * const ccs_mode[CCS_CONFIG_MAX_MODE];
 extern const char * const ccs_path_keyword[CCS_MAX_PATH_OPERATION];
 extern const char * const ccs_proto_keyword[CCS_SOCK_MAX];
@@ -1613,6 +1618,9 @@ static inline void ccs_read_unlock(const int idx)
 
 #else
 
+int ccs_lock(void);
+void ccs_unlock(const int idx);
+
 /**
  * ccs_read_lock - Take lock for protecting policy.
  *
