@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/memory.c
  *
- * Copyright (C) 2005-2010  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.0+   2010/12/31
+ * Version: 1.8.1   2011/04/01
  */
 
 #include "internal.h"
@@ -43,25 +43,27 @@ unsigned int ccs_memory_quota[CCS_MAX_MEMORY_STAT];
 /**
  * ccs_memory_ok - Check memory quota.
  *
- * @ptr:  Pointer to allocated memory.
- * @size: Size in byte.
+ * @ptr:  Pointer to allocated memory. Maybe NULL.
+ * @size: Size in byte. Not used if @ptr is NULL.
  *
  * Returns true if @ptr is not NULL and quota not exceeded, false otherwise.
  */
 bool ccs_memory_ok(const void *ptr, const unsigned int size)
 {
-	size_t s = ccs_round2(size);
-	bool result;
-	spin_lock(&ccs_policy_memory_lock);
-	ccs_memory_used[CCS_MEMORY_POLICY] += s;
-	result = ptr && (!ccs_memory_quota[CCS_MEMORY_POLICY] ||
-			 ccs_memory_used[CCS_MEMORY_POLICY] <=
-			 ccs_memory_quota[CCS_MEMORY_POLICY]);
-	if (!result)
-		ccs_memory_used[CCS_MEMORY_POLICY] -= s;
-	spin_unlock(&ccs_policy_memory_lock);
-	if (result)
-		return true;
+	if (ptr) {
+		const size_t s = ccs_round2(size);
+		bool result;
+		spin_lock(&ccs_policy_memory_lock);
+		ccs_memory_used[CCS_MEMORY_POLICY] += s;
+		result = !ccs_memory_quota[CCS_MEMORY_POLICY] ||
+			ccs_memory_used[CCS_MEMORY_POLICY] <=
+			ccs_memory_quota[CCS_MEMORY_POLICY];
+		if (!result)
+			ccs_memory_used[CCS_MEMORY_POLICY] -= s;
+		spin_unlock(&ccs_policy_memory_lock);
+		if (result)
+			return true;
+	}
 	ccs_warn_oom(__func__);
 	return false;
 }
@@ -428,6 +430,7 @@ void __init ccs_mm_init(void)
 	idx = ccs_read_lock();
 	if (ccs_find_domain(CCS_ROOT_NAME) != &ccs_kernel_domain)
 		panic("Can't register ccs_kernel_domain");
+#ifdef CONFIG_CCSECURITY_BUILTIN_INITIALIZERS
 	{
 		/* Load built-in policy. */
 		static char ccs_builtin_initializers[] __initdata
@@ -443,5 +446,6 @@ void __init ccs_mm_init(void)
 			cp = cp2;
 		}
 	}
+#endif
 	ccs_read_unlock(idx);
 }
