@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/policy_io.c
  *
- * Copyright (C) 2005-2010  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.0   2010/11/11
+ * Version: 1.8.1+   2011/04/11
  */
 
 #include "internal.h"
@@ -257,8 +257,7 @@ const char * const ccs_condition_keyword[CCS_MAX_CONDITION_KEYWORD] = {
 
 /* String table for PREFERENCE keyword. */
 static const char * const ccs_pref_keywords[CCS_MAX_PREF] = {
-	[CCS_PREF_MAX_GRANT_LOG]      = "max_grant_log",
-	[CCS_PREF_MAX_REJECT_LOG]     = "max_reject_log",
+	[CCS_PREF_MAX_AUDIT_LOG]      = "max_audit_log",
 	[CCS_PREF_MAX_LEARNING_ENTRY] = "max_learning_entry",
 	[CCS_PREF_ENFORCING_PENALTY]  = "enforcing_penalty",
 };
@@ -273,12 +272,12 @@ static bool ccs_manage_by_non_root;
  *
  * Returns "yes" if @value is not 0, "no" otherwise.
  */
-static const char *ccs_yesno(const unsigned int value)
+const char *ccs_yesno(const unsigned int value)
 {
 	return value ? "yes" : "no";
 }
 
-/* Prototype fpr ccs_addprintf(). */
+/* Prototype for ccs_addprintf(). */
 static void ccs_addprintf(char *buffer, int len, const char *fmt, ...)
 	__attribute__ ((format(printf, 3, 4)));
 
@@ -311,7 +310,7 @@ static bool ccs_flush(struct ccs_io_buffer *head)
 {
 	while (head->r.w_pos) {
 		const char *w = head->r.w[0];
-		int len = strlen(w);
+		size_t len = strlen(w);
 		if (len) {
 			if (len > head->read_user_buf_avail)
 				len = head->read_user_buf_avail;
@@ -323,10 +322,9 @@ static bool ccs_flush(struct ccs_io_buffer *head)
 			head->read_user_buf += len;
 			w += len;
 		}
-		if (*w) {
-			head->r.w[0] = w;
+		head->r.w[0] = w;
+		if (*w)
 			return false;
-		}
 		/* Add '\0' for audit logs and query. */
 		if (head->poll) {
 			if (!head->read_user_buf_avail ||
@@ -364,6 +362,10 @@ static void ccs_set_string(struct ccs_io_buffer *head, const char *string)
 		printk(KERN_WARNING "Too many words in a line.\n");
 }
 
+/* Prototype for ccs_io_printf(). */
+static void ccs_io_printf(struct ccs_io_buffer *head, const char *fmt, ...)
+	__attribute__ ((format(printf, 2, 3)));
+
 /**
  * ccs_io_printf - printf() to "struct ccs_io_buffer" structure.
  *
@@ -372,11 +374,11 @@ static void ccs_set_string(struct ccs_io_buffer *head, const char *string)
  *
  * Returns nothing.
  */
-void ccs_io_printf(struct ccs_io_buffer *head, const char *fmt, ...)
+static void ccs_io_printf(struct ccs_io_buffer *head, const char *fmt, ...)
 {
 	va_list args;
-	int len;
-	int pos = head->r.avail;
+	size_t len;
+	size_t pos = head->r.avail;
 	int size = head->readbuf_size - pos;
 	if (size <= 0)
 		return;
@@ -416,6 +418,18 @@ static bool ccs_set_lf(struct ccs_io_buffer *head)
 	return !head->r.w_pos;
 }
 
+/**
+ * ccs_set_slash - Put a shash to "struct ccs_io_buffer" structure.
+ *
+ * @head: Pointer to "struct ccs_io_buffer".
+ *
+ * Returns nothing.
+ */
+static void ccs_set_slash(struct ccs_io_buffer *head)
+{
+	ccs_set_string(head, "/");
+}
+
 /**
  * ccs_assign_profile - Create a new profile.
  *
@@ -442,10 +456,8 @@ static struct ccs_profile *ccs_assign_profile(const unsigned int profile)
 			CCS_CONFIG_WANT_GRANT_LOG | CCS_CONFIG_WANT_REJECT_LOG;
 		memset(ptr->config, CCS_CONFIG_USE_DEFAULT,
 		       sizeof(ptr->config));
-		ptr->pref[CCS_PREF_MAX_GRANT_LOG] =
-			CONFIG_CCSECURITY_MAX_GRANT_LOG;
-		ptr->pref[CCS_PREF_MAX_REJECT_LOG] =
-			CONFIG_CCSECURITY_MAX_REJECT_LOG;
+		ptr->pref[CCS_PREF_MAX_AUDIT_LOG] =
+			CONFIG_CCSECURITY_MAX_AUDIT_LOG;
 		ptr->pref[CCS_PREF_MAX_LEARNING_ENTRY] =
 			CONFIG_CCSECURITY_MAX_ACCEPT_ENTRY;
 		mb(); /* Avoid out-of-order execution. */
@@ -472,7 +484,7 @@ static void ccs_check_profile(void)
 		const u8 profile = domain->profile;
 		if (ccs_profile_ptr[profile])
 			continue;
-		printk(KERN_ERR "You need to define profile %u before using it.\n",
+		printk(KERN_ERR "Profile %u must be defined before using it.\n",
 		       profile);
 		printk(KERN_ERR "Please see http://tomoyo.sourceforge.jp/1.8/ "
 		       "for more information.\n");
@@ -481,14 +493,14 @@ static void ccs_check_profile(void)
 	}
 	ccs_read_unlock(idx);
 	if (ccs_profile_version != 20100903) {
-		printk(KERN_ERR "You need to install userland programs for "
-		       "TOMOYO 1.8 and initialize policy configuration.\n");
+		printk(KERN_ERR "Userland tools must be installed for "
+		       "TOMOYO 1.8, and policy must be initialized.\n");
 		printk(KERN_ERR "Please see http://tomoyo.sourceforge.jp/1.8/ "
 		       "for more information.\n");
 		panic("Profile version %u is not supported.\n",
 		      ccs_profile_version);
 	}
-	printk(KERN_INFO "CCSecurity: 1.8.0   2010/11/11\n");
+	printk(KERN_INFO "CCSecurity: 1.8.1+   2011/04/11\n");
 	printk(KERN_INFO "Mandatory Access Control activated.\n");
 }
 
@@ -650,8 +662,15 @@ static int ccs_write_profile(struct ccs_io_buffer *head)
 		return -EINVAL;
 	*cp++ = '\0';
 	if (!strcmp(data, "COMMENT")) {
-		const struct ccs_path_info *old_comment = profile->comment;
-		profile->comment = ccs_get_name(cp);
+		static DEFINE_SPINLOCK(lock);
+		const struct ccs_path_info *new_comment = ccs_get_name(cp);
+		const struct ccs_path_info *old_comment;
+		if (!new_comment)
+			return -ENOMEM;
+		spin_lock(&lock);
+		old_comment = profile->comment;
+		profile->comment = new_comment;
+		spin_unlock(&lock);
 		ccs_put_name(old_comment);
 		return 0;
 	}
@@ -698,7 +717,7 @@ next:
 	profile = ccs_profile_ptr[index];
 	switch (head->r.step) {
 	case 0:
-		ccs_io_printf(head, "PROFILE_VERSION=%s\n", "20100903");
+		ccs_io_printf(head, "PROFILE_VERSION=%u\n", 20100903);
 		head->r.step++;
 		break;
 	case 1:
@@ -920,8 +939,8 @@ static bool ccs_select_one(struct ccs_io_buffer *head, const char *data)
 	unsigned int pid;
 	struct ccs_domain_info *domain = NULL;
 	bool global_pid = false;
-	if (!strcmp(data, "execute")) {
-		head->r.print_execute_only = true;
+	if (!strcmp(data, "transition_only")) {
+		head->r.print_transition_related_only = true;
 		return true;
 	}
 	if (sscanf(data, "pid=%u", &pid) == 1 ||
@@ -1107,7 +1126,7 @@ static int ccs_write_domain(struct ccs_io_buffer *head)
 	else if (ccs_str_starts(&data, "select "))
 		is_select = true;
 	if (is_select && ccs_select_one(head, data))
-		return 0;
+		return -EAGAIN;
 	/* Don't allow updating policies by non manager programs. */
 	if (!ccs_manager())
 		return -EPERM;
@@ -1128,12 +1147,14 @@ static int ccs_write_domain(struct ccs_io_buffer *head)
 	if (sscanf(data, "use_profile %u\n", &profile) == 1
 	    && profile < CCS_MAX_PROFILES) {
 		if (!ccs_policy_loaded || ccs_profile_ptr[(u8) profile])
-			domain->profile = (u8) profile;
+			if (!is_delete)
+				domain->profile = (u8) profile;
 		return 0;
 	}
 	if (sscanf(data, "use_group %u\n", &profile) == 1
 	    && profile < CCS_MAX_ACL_GROUPS) {
-		domain->group = (u8) profile;
+		if (!is_delete)
+			domain->group = (u8) profile;
 		return 0;
 	}
 	for (profile = 0; profile < CCS_MAX_DOMAIN_INFO_FLAGS; profile++) {
@@ -1360,32 +1381,18 @@ static bool ccs_print_condition(struct ccs_io_buffer *head,
 }
 
 /**
- * ccs_fns - Find next set bit.
+ * ccs_set_group - Print "acl_group " header keyword and category name.
  *
- * @perm: 8 bits value.
- * @bit:  First bit to find.
- *
- * Returns next set bit on success, 8 otherwise.
- */
-static u8 ccs_fns(const u8 perm, u8 bit)
-{
-	for ( ; bit < 8; bit++)
-		if (perm & (1 << bit))
-			break;
-	return bit;
-}
-
-/**
- * ccs_set_group - Print "acl_group " header keyword.
- *
- * @head: Pointer to "struct ccs_io_buffer".
+ * @head:     Pointer to "struct ccs_io_buffer".
+ * @category: Category name.
  *
  * Returns nothing.
  */
-static void ccs_set_group(struct ccs_io_buffer *head)
+static void ccs_set_group(struct ccs_io_buffer *head, const char *category)
 {
 	if (head->type == CCS_EXCEPTIONPOLICY)
-		ccs_io_printf(head, "acl_group %u ", head->r.group_index);
+		ccs_io_printf(head, "acl_group %u ", head->r.acl_group_index);
+	ccs_set_string(head, category);
 }
 
 /**
@@ -1400,40 +1407,41 @@ static bool ccs_print_entry(struct ccs_io_buffer *head,
 			    const struct ccs_acl_info *acl)
 {
 	const u8 acl_type = acl->type;
+	const bool may_trigger_transition = acl->cond && acl->cond->transit;
+	bool first = true;
 	u8 bit;
 	if (head->r.print_cond_part)
 		goto print_cond_part;
 	if (acl->is_deleted)
 		return true;
-next:
-	bit = head->r.bit;
 	if (!ccs_flush(head))
 		return false;
 	else if (acl_type == CCS_TYPE_PATH_ACL) {
 		struct ccs_path_acl *ptr
 			= container_of(acl, typeof(*ptr), head);
 		const u16 perm = ptr->perm;
-		for ( ; bit < CCS_MAX_PATH_OPERATION; bit++) {
+		for (bit = 0; bit < CCS_MAX_PATH_OPERATION; bit++) {
 			if (!(perm & (1 << bit)))
 				continue;
-			if (head->r.print_execute_only &&
-			    bit != CCS_TYPE_EXECUTE
-			    /* && bit != CCS_TYPE_TRANSIT */)
+			if (head->r.print_transition_related_only &&
+			    bit != CCS_TYPE_EXECUTE && !may_trigger_transition)
 				continue;
-			break;
+			if (first) {
+				ccs_set_group(head, "file ");
+				first = false;
+			} else {
+				ccs_set_slash(head);
+			}
+			ccs_set_string(head, ccs_path_keyword[bit]);
 		}
-		if (bit >= CCS_MAX_PATH_OPERATION)
-			goto done;
-		ccs_set_group(head);
-		ccs_set_string(head, "file ");
-		ccs_set_string(head, ccs_path_keyword[bit]);
+		if (first)
+			return true;
 		ccs_print_name_union(head, &ptr->name);
 	} else if (acl_type == CCS_TYPE_AUTO_EXECUTE_HANDLER ||
 		   acl_type == CCS_TYPE_DENIED_EXECUTE_HANDLER) {
 		struct ccs_handler_acl *ptr
 			= container_of(acl, typeof(*ptr), head);
-		ccs_set_group(head);
-		ccs_set_string(head, "task ");
+		ccs_set_group(head, "task ");
 		ccs_set_string(head, acl_type == CCS_TYPE_AUTO_EXECUTE_HANDLER
 			       ? "auto_execute_handler " :
 			       "denied_execute_handler ");
@@ -1442,23 +1450,32 @@ next:
 		   acl_type == CCS_TYPE_MANUAL_TASK_ACL) {
 		struct ccs_task_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		ccs_set_group(head);
-		ccs_set_string(head, "task ");
+		ccs_set_group(head, "task ");
 		ccs_set_string(head, acl_type == CCS_TYPE_AUTO_TASK_ACL ?
 			       "auto_domain_transition " :
 			       "manual_domain_transition ");
 		ccs_set_string(head, ptr->domainname->name);
-	} else if (head->r.print_execute_only) {
+	} else if (head->r.print_transition_related_only &&
+		   !may_trigger_transition) {
 		return true;
 	} else if (acl_type == CCS_TYPE_MKDEV_ACL) {
 		struct ccs_mkdev_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		bit = ccs_fns(ptr->perm, bit);
-		if (bit >= CCS_MAX_MKDEV_OPERATION)
-			goto done;
-		ccs_set_group(head);
-		ccs_set_string(head, "file ");
-		ccs_set_string(head, ccs_mac_keywords[ccs_pnnn2mac[bit]]);
+		const u8 perm = ptr->perm;
+		for (bit = 0; bit < CCS_MAX_MKDEV_OPERATION; bit++) {
+			if (!(perm & (1 << bit)))
+				continue;
+			if (first) {
+				ccs_set_group(head, "file ");
+				first = false;
+			} else {
+				ccs_set_slash(head);
+			}
+			ccs_set_string(head, ccs_mac_keywords
+				       [ccs_pnnn2mac[bit]]);
+		}
+		if (first)
+			return true;
 		ccs_print_name_union(head, &ptr->name);
 		ccs_print_number_union(head, &ptr->mode);
 		ccs_print_number_union(head, &ptr->major);
@@ -1466,49 +1483,74 @@ next:
 	} else if (acl_type == CCS_TYPE_PATH2_ACL) {
 		struct ccs_path2_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		bit = ccs_fns(ptr->perm, bit);
-		if (bit >= CCS_MAX_PATH2_OPERATION)
-			goto done;
-		ccs_set_group(head);
-		ccs_set_string(head, "file ");
-		ccs_set_string(head, ccs_mac_keywords[ccs_pp2mac[bit]]);
+		const u8 perm = ptr->perm;
+		for (bit = 0; bit < CCS_MAX_PATH2_OPERATION; bit++) {
+			if (!(perm & (1 << bit)))
+				continue;
+			if (first) {
+				ccs_set_group(head, "file ");
+				first = false;
+			} else {
+				ccs_set_slash(head);
+			}
+			ccs_set_string(head, ccs_mac_keywords
+				       [ccs_pp2mac[bit]]);
+		}
+		if (first)
+			return true;
 		ccs_print_name_union(head, &ptr->name1);
 		ccs_print_name_union(head, &ptr->name2);
 	} else if (acl_type == CCS_TYPE_PATH_NUMBER_ACL) {
 		struct ccs_path_number_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		bit = ccs_fns(ptr->perm, bit);
-		if (bit >= CCS_MAX_PATH_NUMBER_OPERATION)
-			goto done;
-		ccs_set_group(head);
-		ccs_set_string(head, "file ");
-		ccs_set_string(head, ccs_mac_keywords[ccs_pn2mac[bit]]);
+		const u8 perm = ptr->perm;
+		for (bit = 0; bit < CCS_MAX_PATH_NUMBER_OPERATION; bit++) {
+			if (!(perm & (1 << bit)))
+				continue;
+			if (first) {
+				ccs_set_group(head, "file ");
+				first = false;
+			} else {
+				ccs_set_slash(head);
+			}
+			ccs_set_string(head, ccs_mac_keywords
+				       [ccs_pn2mac[bit]]);
+		}
+		if (first)
+			return true;
 		ccs_print_name_union(head, &ptr->name);
 		ccs_print_number_union(head, &ptr->number);
 	} else if (acl_type == CCS_TYPE_ENV_ACL) {
 		struct ccs_env_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		ccs_set_group(head);
-		ccs_set_string(head, "misc env ");
+		ccs_set_group(head, "misc env ");
 		ccs_set_string(head, ptr->env->name);
 	} else if (acl_type == CCS_TYPE_CAPABILITY_ACL) {
 		struct ccs_capability_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		ccs_set_group(head);
-		ccs_set_string(head, "capability ");
-		ccs_set_string(head,
-			       ccs_mac_keywords[ccs_c2mac[ptr->operation]]);
+		ccs_set_group(head, "capability ");
+		ccs_set_string(head, ccs_mac_keywords
+			       [ccs_c2mac[ptr->operation]]);
 	} else if (acl_type == CCS_TYPE_INET_ACL) {
 		struct ccs_inet_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		bit = ccs_fns(ptr->perm, bit);
-		if (bit >= CCS_MAX_NETWORK_OPERATION)
-			goto done;
-		ccs_set_group(head);
-		ccs_set_string(head, "network inet ");
-		ccs_set_string(head, ccs_proto_keyword[ptr->protocol]);
-		ccs_set_space(head);
-		ccs_set_string(head, ccs_socket_keyword[bit]);
+		const u8 perm = ptr->perm;
+		for (bit = 0; bit < CCS_MAX_NETWORK_OPERATION; bit++) {
+			if (!(perm & (1 << bit)))
+				continue;
+			if (first) {
+				ccs_set_group(head, "network inet ");
+				ccs_set_string(head, ccs_proto_keyword
+					       [ptr->protocol]);
+				ccs_set_space(head);
+				first = false;
+			} else {
+				ccs_set_slash(head);
+			}
+			ccs_set_string(head, ccs_socket_keyword[bit]);
+		}
+		if (first)
+			return true;
 		ccs_set_space(head);
 		switch (ptr->address_type) {
 			char buf[128];
@@ -1532,33 +1574,39 @@ next:
 	} else if (acl_type == CCS_TYPE_UNIX_ACL) {
 		struct ccs_unix_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		bit = ccs_fns(ptr->perm, bit);
-		if (bit >= CCS_MAX_NETWORK_OPERATION)
-			goto done;
-		ccs_set_group(head);
-		ccs_set_string(head, "network unix ");
-		ccs_set_string(head, ccs_proto_keyword[ptr->protocol]);
-		ccs_set_space(head);
-		ccs_set_string(head, ccs_socket_keyword[bit]);
+		const u8 perm = ptr->perm;
+		for (bit = 0; bit < CCS_MAX_NETWORK_OPERATION; bit++) {
+			if (!(perm & (1 << bit)))
+				continue;
+			if (first) {
+				ccs_set_group(head, "network unix ");
+				ccs_set_string(head, ccs_proto_keyword
+					       [ptr->protocol]);
+				ccs_set_space(head);
+				first = false;
+			} else {
+				ccs_set_slash(head);
+			}
+			ccs_set_string(head, ccs_socket_keyword[bit]);
+		}
+		if (first)
+			return true;
 		ccs_print_name_union(head, &ptr->name);
 	} else if (acl_type == CCS_TYPE_SIGNAL_ACL) {
 		struct ccs_signal_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		ccs_set_group(head);
-		ccs_set_string(head, "ipc signal ");
+		ccs_set_group(head, "ipc signal ");
 		ccs_io_printf(head, "%u ", ptr->sig);
 		ccs_set_string(head, ptr->domainname->name);
 	} else if (acl_type == CCS_TYPE_MOUNT_ACL) {
 		struct ccs_mount_acl *ptr =
 			container_of(acl, typeof(*ptr), head);
-		ccs_set_group(head);
-		ccs_io_printf(head, "file mount");
+		ccs_set_group(head, "file mount");
 		ccs_print_name_union(head, &ptr->dev_name);
 		ccs_print_name_union(head, &ptr->dir_name);
 		ccs_print_name_union(head, &ptr->fs_type);
 		ccs_print_number_union(head, &ptr->flags);
 	}
-	head->r.bit = bit + 1;
 	if (acl->cond) {
 		head->r.print_cond_part = true;
 		head->r.cond_step = 0;
@@ -1571,17 +1619,6 @@ print_cond_part:
 	} else {
 		ccs_set_lf(head);
 	}
-	switch (acl_type) {
-	case CCS_TYPE_PATH_ACL:
-	case CCS_TYPE_MKDEV_ACL:
-	case CCS_TYPE_PATH2_ACL:
-	case CCS_TYPE_PATH_NUMBER_ACL:
-	case CCS_TYPE_INET_ACL:
-	case CCS_TYPE_UNIX_ACL:
-		goto next;
-	}
-done:
-	head->r.bit = 0;
 	return true;
 }
 
@@ -1840,6 +1877,14 @@ static int ccs_write_exception(struct ccs_io_buffer *head)
 		{ "aggregator ",    ccs_write_aggregator },
 		{ "deny_autobind ", ccs_write_reserved_port },
 	};
+	if (!is_delete && ccs_str_starts(&data, "select ") &&
+	    !strcmp(data, "transition_only")) {
+		head->r.print_transition_related_only = true;
+		return -EAGAIN;
+	}
+	/* Don't allow updating policies by non manager programs. */
+	if (!ccs_manager())
+		return -EPERM;
 	for (i = 0; i < 2; i++)
 		if (ccs_str_starts(&data, ccs_callback[i].keyword))
 			return ccs_callback[i].write(data, is_delete);
@@ -1937,6 +1982,9 @@ static bool ccs_read_policy(struct ccs_io_buffer *head, const int idx)
 			container_of(head->r.acl, typeof(*acl), list);
 		if (acl->is_deleted)
 			continue;
+		if (head->r.print_transition_related_only &&
+		    idx != CCS_ID_TRANSITION_CONTROL)
+			continue;
 		if (!ccs_flush(head))
 			return false;
 		switch (idx) {
@@ -2010,10 +2058,10 @@ static void ccs_read_exception(struct ccs_io_buffer *head)
 		return;
 	while (head->r.step < CCS_MAX_POLICY + CCS_MAX_GROUP
 	       + CCS_MAX_ACL_GROUPS * 2) {
-		head->r.group_index = (head->r.step - CCS_MAX_POLICY
-				       - CCS_MAX_GROUP) / 2;
+		head->r.acl_group_index = (head->r.step - CCS_MAX_POLICY
+					   - CCS_MAX_GROUP) / 2;
 		if (!ccs_read_domain2(head,
-				      &ccs_acl_group[head->r.group_index],
+				      &ccs_acl_group[head->r.acl_group_index],
 				      head->r.step & 1))
 			return;
 		head->r.step++;
@@ -2030,10 +2078,10 @@ static DECLARE_WAIT_QUEUE_HEAD(ccs_answer_wait);
 struct ccs_query {
 	struct list_head list;
 	char *query;
-	int query_len;
+	size_t query_len;
 	unsigned int serial;
-	int timer;
-	int answer;
+	u8 timer;
+	u8 answer;
 	u8 retry;
 };
 
@@ -2144,7 +2192,7 @@ int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
 	va_start(args, fmt);
 	len = vsnprintf((char *) &len, 1, fmt, args) + 1;
 	va_end(args);
-	/* Write /proc/ccs/grant_log or /proc/ccs/reject_log. */
+	/* Write /proc/ccs/audit. */
 	va_start(args, fmt);
 	ccs_write_log2(r, len, fmt, args);
 	va_end(args);
@@ -2160,7 +2208,7 @@ int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
 		error = -EPERM;
 		if (atomic_read(&ccs_query_observers))
 			break;
-		if (ccs_current_flags() & CCS_DONT_SLEEP_ON_ENFORCE_ERROR)
+		if (r->dont_sleep_on_enforce_error)
 			goto out;
 		p = ccs_profile(r->profile);
 		/* Check enforcing_penalty parameter. */
@@ -2171,7 +2219,7 @@ int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
 		goto out;
 	case CCS_CONFIG_LEARNING:
 		error = 0;
-		/* Check mac_learning_entry parameter. */
+		/* Check max_learning_entry parameter. */
 		if (ccs_domain_quota_ok(r))
 			break;
 		/* fall through */
@@ -2281,15 +2329,13 @@ static int ccs_poll_query(struct file *file, poll_table *wait)
 static void ccs_read_query(struct ccs_io_buffer *head)
 {
 	struct list_head *tmp;
-	int pos = 0;
-	int len = 0;
+	unsigned int pos = 0;
+	size_t len = 0;
 	char *buf;
 	if (head->r.w_pos)
 		return;
-	if (head->read_buf) {
-		kfree(head->read_buf);
-		head->read_buf = NULL;
-	}
+	kfree(head->read_buf);
+	head->read_buf = NULL;
 	spin_lock(&ccs_query_list_lock);
 	list_for_each(tmp, &ccs_query_list) {
 		struct ccs_query *ptr = list_entry(tmp, typeof(*ptr), list);
@@ -2362,7 +2408,7 @@ static int ccs_write_answer(struct ccs_io_buffer *head)
 		if (ptr->serial != serial)
 			continue;
 		if (!ptr->answer)
-			ptr->answer = answer;
+			ptr->answer = (u8) answer;
 		break;
 	}
 	spin_unlock(&ccs_query_list_lock);
@@ -2381,11 +2427,11 @@ static void ccs_read_version(struct ccs_io_buffer *head)
 {
 	if (head->r.eof)
 		return;
-	ccs_set_string(head, "1.8.0");
+	ccs_set_string(head, "1.8.1");
 	head->r.eof = true;
 }
 
-/* String table for /proc/ccs/meminfo interface. */
+/* String table for /proc/ccs/stat interface. */
 static const char * const ccs_policy_headers[CCS_MAX_POLICY_STAT] = {
 	[CCS_STAT_POLICY_UPDATES]    = "update:",
 	[CCS_STAT_POLICY_LEARNING]   = "violation in learning mode:",
@@ -2393,7 +2439,7 @@ static const char * const ccs_policy_headers[CCS_MAX_POLICY_STAT] = {
 	[CCS_STAT_POLICY_ENFORCING]  = "violation in enforcing mode:",
 };
 
-/* String table for /proc/ccs/meminfo interface. */
+/* String table for /proc/ccs/stat interface. */
 static const char * const ccs_memory_headers[CCS_MAX_MEMORY_STAT] = {
 	[CCS_MEMORY_POLICY] = "policy:",
 	[CCS_MEMORY_AUDIT]  = "audit log:",
@@ -2482,57 +2528,6 @@ static int ccs_write_stat(struct ccs_io_buffer *head)
 	return 0;
 }
 
-/* String table for /proc/ccs/meminfo interface. */
-static const char * const ccs_old_memory_header[CCS_MAX_MEMORY_STAT] = {
-	[CCS_MEMORY_POLICY] = "Policy:",
-	[CCS_MEMORY_AUDIT]  = "Audit logs:",
-	[CCS_MEMORY_QUERY]  = "Query lists:",
-};
-
-/**
- * ccs_read_memory_counter - Read memory usage.
- *
- * @head: Pointer to "struct ccs_io_buffer".
- *
- * Returns nothing.
- */
-static void ccs_read_memory_counter(struct ccs_io_buffer *head)
-{
-	unsigned int total = 0;
-	int i;
-	if (head->r.eof)
-		return;
-	for (i = 0; i < CCS_MAX_MEMORY_STAT; i++) {
-		unsigned int used = ccs_memory_used[i];
-		total += used;
-		ccs_io_printf(head, "%-12s %10u", ccs_old_memory_header[i],
-			      used);
-		if (ccs_memory_quota[i])
-			ccs_io_printf(head, "   (Quota: %10u)",
-				      ccs_memory_quota[i]);
-		ccs_io_printf(head, "\n");
-	}
-	ccs_io_printf(head, "%-12s %10u\n", "Total:", total);
-	head->r.eof = true;
-}
-
-/**
- * ccs_write_memory_quota - Set memory quota.
- *
- * @head: Pointer to "struct ccs_io_buffer".
- *
- * Returns 0.
- */
-static int ccs_write_memory_quota(struct ccs_io_buffer *head)
-{
-	char *data = head->write_buf;
-	u8 i;
-	for (i = 0; i < CCS_MAX_MEMORY_STAT; i++)
-		if (ccs_str_starts(&data, ccs_old_memory_header[i]))
-			sscanf(data, "%u", &ccs_memory_quota[i]);
-	return 0;
-}
-
 /**
  * ccs_open_control - open() for /proc/ccs/ interface.
  *
@@ -2557,8 +2552,7 @@ int ccs_open_control(const u8 type, struct file *file)
 		head->write = ccs_write_exception;
 		head->read = ccs_read_exception;
 		break;
-	case CCS_GRANTLOG: /* /proc/ccs/grant_log */
-	case CCS_REJECTLOG: /* /proc/ccs/reject_log */
+	case CCS_AUDIT: /* /proc/ccs/audit */
 		head->poll = ccs_poll_log;
 		head->read = ccs_read_log;
 		break;
@@ -2586,11 +2580,6 @@ int ccs_open_control(const u8 type, struct file *file)
 		head->read = ccs_read_stat;
 		head->readbuf_size = 1024;
 		break;
-	case CCS_MEMINFO: /* /proc/ccs/meminfo */
-		head->write = ccs_write_memory_quota;
-		head->read = ccs_read_memory_counter;
-		head->readbuf_size = 512;
-		break;
 	case CCS_PROFILE: /* /proc/ccs/profile */
 		head->write = ccs_write_profile;
 		head->read = ccs_read_profile;
@@ -2644,11 +2633,8 @@ int ccs_open_control(const u8 type, struct file *file)
 	 */
 	if (type == CCS_QUERY)
 		atomic_inc(&ccs_query_observers);
-	else if (type != CCS_GRANTLOG && type != CCS_REJECTLOG &&
-		 type != CCS_VERSION && type != CCS_MEMINFO &&
-		 type != CCS_STAT)
-		head->reader_idx = ccs_lock();
 	file->private_data = head;
+	ccs_notify_gc(head, true);
 	return 0;
 }
 
@@ -2662,8 +2648,7 @@ int ccs_open_control(const u8 type, struct file *file)
  *
  * Waits for read readiness.
  * /proc/ccs/query is handled by /usr/sbin/ccs-queryd and
- * /proc/ccs/grant_log and /proc/ccs/reject_log are handled by
- * /usr/sbin/ccs-auditd.
+ * /proc/ccs/audit is handled by /usr/sbin/ccs-auditd.
  */
 int ccs_poll_control(struct file *file, poll_table *wait)
 {
@@ -2682,8 +2667,8 @@ int ccs_poll_control(struct file *file, poll_table *wait)
  *
  * Returns bytes read on success, negative value otherwise.
  */
-int ccs_read_control(struct file *file, char __user *buffer,
-		     const int buffer_len)
+ssize_t ccs_read_control(struct file *file, char __user *buffer,
+			 const size_t buffer_len)
 {
 	int len;
 	struct ccs_io_buffer *head = file->private_data;
@@ -2716,12 +2701,12 @@ int ccs_read_control(struct file *file, char __user *buffer,
  *
  * Returns @buffer_len on success, negative value otherwise.
  */
-int ccs_write_control(struct file *file, const char __user *buffer,
-		      const int buffer_len)
+ssize_t ccs_write_control(struct file *file, const char __user *buffer,
+			  const size_t buffer_len)
 {
 	struct ccs_io_buffer *head = file->private_data;
 	int error = buffer_len;
-	int avail_len = buffer_len;
+	size_t avail_len = buffer_len;
 	char *cp0 = head->write_buf;
 	int idx;
 	if (!head->write)
@@ -2733,7 +2718,7 @@ int ccs_write_control(struct file *file, const char __user *buffer,
 	idx = ccs_read_lock();
 	/* Don't allow updating policies by non manager programs. */
 	if (head->write != ccs_write_pid && head->write != ccs_write_domain &&
-	    !ccs_manager()) {
+	    head->write != ccs_write_exception && !ccs_manager()) {
 		ccs_read_unlock(idx);
 		mutex_unlock(&head->io_sem);
 		return -EPERM;
@@ -2766,16 +2751,26 @@ int ccs_write_control(struct file *file, const char __user *buffer,
 		cp0[head->w.avail - 1] = '\0';
 		head->w.avail = 0;
 		ccs_normalize_line(cp0);
-		if (head->write(head))
-			continue;
+		{
+			const int ret = head->write(head);
+			if (ret == -EPERM) {
+				error = -EPERM;
+				break;
+			}
+			if (ret)
+				continue;
+		}
 		switch (head->type) {
 		case CCS_DOMAINPOLICY:
 		case CCS_EXCEPTIONPOLICY:
 		case CCS_DOMAIN_STATUS:
-		case CCS_MEMINFO:
+		case CCS_STAT:
 		case CCS_PROFILE:
 		case CCS_MANAGER:
 			ccs_update_stat(CCS_STAT_POLICY_UPDATES);
+			break;
+		default:
+			break;
 		}
 	}
 	ccs_read_unlock(idx);
@@ -2793,28 +2788,14 @@ int ccs_write_control(struct file *file, const char __user *buffer,
 int ccs_close_control(struct file *file)
 {
 	struct ccs_io_buffer *head = file->private_data;
-	const bool is_write = head->write_buf != NULL;
-	const u8 type = head->type;
+	file->private_data = NULL;
 	/*
 	 * If the file is /proc/ccs/query, decrement the observer counter.
 	 */
-	if (type == CCS_QUERY) {
-		if (atomic_dec_and_test(&ccs_query_observers))
-			wake_up_all(&ccs_answer_wait);
-	} else if (type != CCS_GRANTLOG && type != CCS_REJECTLOG &&
-		   type != CCS_VERSION && type != CCS_MEMINFO &&
-		   type != CCS_STAT)
-		ccs_unlock(head->reader_idx);
-	/* Release memory used for policy I/O. */
-	kfree(head->read_buf);
-	head->read_buf = NULL;
-	kfree(head->write_buf);
-	head->write_buf = NULL;
-	kfree(head);
-	head = NULL;
-	file->private_data = NULL;
-	if (is_write)
-		ccs_run_gc();
+	if (head->type == CCS_QUERY &&
+	    atomic_dec_and_test(&ccs_query_observers))
+		wake_up_all(&ccs_answer_wait);
+	ccs_notify_gc(head, false);
 	return 0;
 }
 
@@ -2827,3 +2808,64 @@ void __init ccs_policy_io_init(void)
 {
 	ccsecurity_ops.check_profile = ccs_check_profile;
 }
+
+#ifdef CONFIG_CCSECURITY_USE_BUILTIN_POLICY
+/**
+ * ccs_load_builtin_policy - Load built-in policy.
+ *
+ * Returns nothing.
+ */
+void __init ccs_load_builtin_policy(void)
+{
+	/*
+	 * This include file is manually created and contains built-in policy
+	 * named "ccs_builtin_profile", "ccs_builtin_exception_policy",
+	 * "ccs_builtin_domain_policy", "ccs_builtin_manager",
+	 * "ccs_builtin_stat" in the form of "static char [] __initdata".
+	 */
+#include "builtin-policy.h"
+	struct ccs_io_buffer head;
+	u8 i;
+	const int idx = ccs_read_lock();
+	for (i = 0; i < 5; i++) {
+		char *start = "";
+		memset(&head, 0, sizeof(head));
+		switch (i) {
+		case 0:
+			start = ccs_builtin_profile;
+			head.write = ccs_write_profile;
+			break;
+		case 1:
+			start = ccs_builtin_exception_policy;
+			head.write = ccs_write_exception;
+			break;
+		case 2:
+			start = ccs_builtin_domain_policy;
+			head.write = ccs_write_domain;
+			break;
+		case 3:
+			start = ccs_builtin_manager;
+			head.write = ccs_write_manager;
+			break;
+		case 4:
+			start = ccs_builtin_stat;
+			head.write = ccs_write_stat;
+			break;
+		}
+		while (1) {
+			char *end = strchr(start, '\n');
+			if (!end)
+				break;
+			*end = '\0';
+			ccs_normalize_line(start);
+			head.write_buf = start;
+			head.write(&head);
+			start = end + 1;
+		}
+	}
+	ccs_read_unlock(idx);
+#ifdef CONFIG_CCSECURITY_ACTIVATE_FROM_THE_BEGINNING
+	ccs_check_profile();
+#endif
+}
+#endif
